"""
Views for the ``pending-enterprise-customer-admin-user`` API endpoint.
"""

from django_filters.rest_framework import DjangoFilterBackend
from edx_rbac.mixins import PermissionRequiredMixin
from rest_framework import filters, permissions

from enterprise import models
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.constants import PENDING_ENT_CUSTOMER_ADMIN_PROVISIONING_ADMIN_ACCESS_PERMISSION
from enterprise.logging import getEnterpriseLogger

LOGGER = getEnterpriseLogger(__name__)


class PendingEnterpriseCustomerAdminUserViewSet(PermissionRequiredMixin, EnterpriseReadWriteModelViewSet):
    """
    API views for the ``pending-enterprise-customer-admin-user`` API endpoint.
    Requires staff permissions
    """
    queryset = models.PendingEnterpriseCustomerAdminUser.objects.all()
    filter_backends = (filters.OrderingFilter, DjangoFilterBackend)
    serializer_class = serializers.PendingEnterpriseCustomerAdminUserSerializer
    permission_classes = (permissions.IsAuthenticated,)
    permission_required = PENDING_ENT_CUSTOMER_ADMIN_PROVISIONING_ADMIN_ACCESS_PERMISSION

    FIELDS = (
        'enterprise_customer', 'user_email',
    )
    filterset_fields = FIELDS
    ordering_fields = FIELDS



# Generated by Django 4.2.13 on 2024-08-13 14:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0219_alter_enterprisecatalogquery_content_filter'),
    ]

    operations = [
        migrations.AlterField(
            model_name='updateroleassignmentswithcustomersconfig',
            name='role',
            field=models.CharField(blank=True, choices=[('enterprise_admin', 'enterprise_admin'), ('enterprise_learner', 'enterprise_learner'), ('enterprise_catalog_admin', 'enterprise_catalog_admin'), ('enterprise_provisioning_admin', 'enterprise_provisioning_admin'), ('enterprise_openedx_operator', 'enterprise_openedx_operator')], help_text='Specifies which user role assignments to update.  If unspecified, will update for all roles.', max_length=100),
        ),
    ]



"""
Management command for assigning enterprise_learner roles
to existing linked enterprise users that are missing them.
"""

import logging
from time import sleep

from django.contrib import auth
from django.core.management.base import BaseCommand

from enterprise.constants import ENTERPRISE_LEARNER_ROLE
from enterprise.models import EnterpriseCustomerUser, SystemWideEnterpriseRole, SystemWideEnterpriseUserRoleAssignment
from enterprise.utils import batch

log = logging.getLogger(__name__)
User = auth.get_user_model()


class Command(BaseCommand):
    """
    Management command for assigning enterprise_learner roles to existing enterprise users.

    Example usage:
        $ ./manage.py backfill_learner_role_assignments
    """
    help = 'Assigns enterprise_learner role to linked enterprise users missing them.'

    def add_arguments(self, parser):
        """
        Entry point for subclassed commands to add custom arguments.
        """
        parser.add_argument(
            '--batch-limit',
            action='store',
            dest='batch_limit',
            default=250,
            help='Number of records in each batch.',
            type=int,
        )

        parser.add_argument(
            '--batch-sleep',
            action='store',
            dest='batch_sleep',
            default=5,
            help='How long to sleep between batches.',
            type=int,
        )

    def backfill_learner_role_assignments(self, options):
        """
        Assigns enterprise_learner role to users.
        """
        batch_limit = options['batch_limit']
        batch_sleep = options['batch_sleep']

        role = SystemWideEnterpriseRole.objects.get(name=ENTERPRISE_LEARNER_ROLE)
        ecus = EnterpriseCustomerUser.objects.select_related('enterprise_customer').filter(linked=True)

        for ecu_batch in batch(ecus, batch_size=batch_limit):
            for ecu in ecu_batch:
                log.info('Processing EnterpriseCustomerUser %s', ecu)

                user = User.objects.get(id=ecu.user_id)
                enterprise_customer = ecu.enterprise_customer
                role_assignment, created = SystemWideEnterpriseUserRoleAssignment.objects.get_or_create(
                    user=user,
                    role=role,
                    enterprise_customer=enterprise_customer
                )
                if created:
                    log.info(
                        'Created SystemWideEnterpriseUserRoleAssignment %s for enterprise user %s',
                        role_assignment, ecu
                    )
                else:
                    log.info(
                        'Did not create role assignment for enterprise user %s', ecu
                    )

            sleep(batch_sleep)

    def handle(self, *args, **options):
        """
        Entry point for management command execution.
        """
        log.info('Starting assigning enterprise_learner roles to users!')

        self.backfill_learner_role_assignments(options)

        log.info('Successfully finished assigning enterprise_learner roles to users!')



"""
Views for the ``enterprise-customer-branding`` API endpoint.
"""

from edx_rbac.decorators import permission_required
from rest_framework import permissions, status
from rest_framework.decorators import action
from rest_framework.parsers import FormParser, MultiPartParser
from rest_framework.response import Response

from enterprise import models
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.logging import getEnterpriseLogger

LOGGER = getEnterpriseLogger(__name__)


class EnterpriseCustomerBrandingConfigurationViewSet(EnterpriseReadWriteModelViewSet):
    """
    API views for the ``enterprise-customer-branding`` API endpoint.
    """
    permission_classes = (permissions.IsAuthenticated,)
    parser_classes = [MultiPartParser, FormParser]
    queryset = models.EnterpriseCustomerBrandingConfiguration.objects.all()
    serializer_class = serializers.EnterpriseCustomerBrandingConfigurationSerializer

    USER_ID_FILTER = 'enterprise_customer__enterprise_customer_users__user_id'
    FIELDS = (
        'enterprise_customer__slug',
    )
    filterset_fields = FIELDS
    ordering_fields = FIELDS
    lookup_field = 'enterprise_customer__slug'

    @action(methods=['patch'], detail=False, permission_classes=[permissions.IsAuthenticated])
    @permission_required('enterprise.can_access_admin_dashboard', fn=lambda request, enterprise_uuid: enterprise_uuid)
    def update_branding(self, request, enterprise_uuid):
        """
        PATCH /enterprise/api/v1/enterprise-customer-branding/update_branding/uuid

        Requires enterprise customer uuid path parameter
        """
        try:
            enterprise_customer = models.EnterpriseCustomer.objects.get(uuid=enterprise_uuid)
            branding_configs = models.EnterpriseCustomerBrandingConfiguration.objects.filter(
                enterprise_customer=enterprise_customer)
            if len(branding_configs) > 0:
                branding_config = models.EnterpriseCustomerBrandingConfiguration.objects.get(
                    enterprise_customer=enterprise_customer)
            else:
                branding_config = models.EnterpriseCustomerBrandingConfiguration(
                    enterprise_customer=enterprise_customer)

            if 'logo' in request.data:
                branding_config.logo = request.data['logo']
            if 'primary_color' in request.data:
                branding_config.primary_color = request.data['primary_color']
            if 'secondary_color' in request.data:
                branding_config.secondary_color = request.data['secondary_color']
            if 'tertiary_color' in request.data:
                branding_config.tertiary_color = request.data['tertiary_color']
            branding_config.save()
        except Exception:  # pylint: disable=broad-except
            LOGGER.exception(
                'Error with updating branding configuration'
            )
            return Response("Error with updating branding configuration", status=status.HTTP_500_INTERNAL_SERVER_ERROR)
        return Response("Branding was updated", status=status.HTTP_204_NO_CONTENT)



# Generated by Django 3.2.12 on 2022-07-18 21:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0014_genericenterprisecustomerpluginconfiguration_dry_run_mode_enabled'),
    ]

    operations = [
        migrations.RenameField(
            model_name='contentmetadataitemtransmission',
            old_name='deleted_at',
            new_name='remote_deleted_at'
        ),
        migrations.AlterField(
            model_name='contentmetadataitemtransmission',
            name='remote_deleted_at',
            field=models.DateTimeField(blank=True, help_text='Date when the content transmission was deleted in the remote API', null=True),
        ),
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='api_response_body',
            field=models.TextField(blank=True, help_text='The most recent remote API call response body', null=True),
        ),
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='api_response_status_code',
            field=models.PositiveIntegerField(blank=True, help_text='The most recent remote API call response HTTP status code', null=True),
        ),
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='remote_created_at',
            field=models.DateTimeField(blank=True, help_text='Date when the content transmission was created in the remote API', null=True),
        ),
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='remote_updated_at',
            field=models.DateTimeField(blank=True, help_text='Date when the content transmission was last updated in the remote API', null=True),
        ),
    ]



# Generated by Django 3.2.12 on 2022-07-28 14:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0161_alter_enterprisecustomerreportingconfiguration_data_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_executive_education_2U_fulfillment',
            field=models.BooleanField(default=False, help_text='Specifies whether the enterprise should have access to Executive Education (2U) content.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_executive_education_2U_fulfillment',
            field=models.BooleanField(default=False, help_text='Specifies whether the enterprise should have access to Executive Education (2U) content.'),
        ),
    ]



# Generated by Django 3.2.20 on 2023-09-26 16:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0186_auto_20230921_1828'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomerssoconfiguration',
            name='display_name',
            field=models.CharField(blank=True, help_text='The display name of the SSO configuration.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomerssoconfiguration',
            name='display_name',
            field=models.CharField(blank=True, help_text='The display name of the SSO configuration.', max_length=255, null=True),
        ),
    ]



"""
Views for the Enterprise Customer Reporting API.
"""

from edx_rbac.decorators import permission_required
from edx_rest_framework_extensions.auth.jwt.authentication import JwtAuthentication
from rest_framework import permissions, status
from rest_framework.authentication import SessionAuthentication
from rest_framework.response import Response
from rest_framework.status import HTTP_200_OK, HTTP_404_NOT_FOUND
from rest_framework.views import APIView

from enterprise import models
from enterprise.api.utils import get_ent_cust_from_report_config_uuid, get_enterprise_customer_from_user_id
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadWriteModelViewSet
from enterprise.utils import get_enterprise_customer


class EnterpriseCustomerReportingConfigurationViewSet(EnterpriseReadWriteModelViewSet):
    """
    API views for the ``enterprise-customer-reporting`` API endpoint.
    """

    queryset = models.EnterpriseCustomerReportingConfiguration.objects.all()
    serializer_class = serializers.EnterpriseCustomerReportingConfigurationSerializer
    lookup_field = 'uuid'
    permission_classes = [permissions.IsAuthenticated]

    USER_ID_FILTER = 'enterprise_customer__enterprise_customer_users__user_id'
    FIELDS = (
        'enterprise_customer',
    )
    filterset_fields = FIELDS
    ordering_fields = FIELDS

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: get_ent_cust_from_report_config_uuid(kwargs['uuid']))
    def retrieve(self, request, *args, **kwargs):
        return super().retrieve(request, *args, **kwargs)

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: get_enterprise_customer_from_user_id(request.user.id))
    def list(self, request, *args, **kwargs):
        return super().list(request, *args, **kwargs)

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: request.data.get('enterprise_customer_id'))
    def create(self, request, *args, **kwargs):
        config_data = request.data.copy()
        serializer = self.get_serializer(data=config_data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        return Response(serializer.data, status=status.HTTP_201_CREATED)

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: get_ent_cust_from_report_config_uuid(kwargs['uuid']))
    def update(self, request, *args, **kwargs):
        return super().update(request, *args, **kwargs)

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: get_ent_cust_from_report_config_uuid(kwargs['uuid']))
    def partial_update(self, request, *args, **kwargs):
        return super().partial_update(request, *args, **kwargs)

    @permission_required(
        'enterprise.can_manage_reporting_config',
        fn=lambda request, *args, **kwargs: get_ent_cust_from_report_config_uuid(kwargs['uuid']))
    def destroy(self, request, *args, **kwargs):
        return super().destroy(request, *args, **kwargs)


class EnterpriseCustomerReportTypesView(APIView):
    """
    API for getting the report types associated with an enterprise customer
    """
    authentication_classes = [JwtAuthentication, SessionAuthentication]
    permission_classes = [permissions.IsAuthenticated]
    http_method_names = ['get']

    @staticmethod
    def _get_data_types_with_recent_progress_type(data_types):
        """
        Get the data types with only the most recent 'progress' type version

        Arguments:
            data_types (list<str, str>): List of data type tuples.

        Returns:
            (list<str, str>): List of data type tuples with only the most recent 'progress' type.
            e.g. [ ... ('progress', 'progress_v3')]
        """
        progress_data_types = [data_type for data_type in data_types if data_type[1].startswith('progress')]
        progress_data_types.sort(key=lambda data_type: data_type[1])
        data_types_for_frontend = [data_type for data_type in data_types if not data_type[1].startswith('progress')]
        data_types_for_frontend.append((progress_data_types[-1][1], 'progress'))
        return data_types_for_frontend

    @staticmethod
    def _get_data_types_for_non_pearson_customers(data_types):
        """
        Get the data types for non-pearson customers

        Arguments:
            data_types (list<str, str>): List of data type tuples.

        Returns:
            (list<str, str>): List of data type tuples without the Pearson specific types.
        """
        reduced_data_types = []
        for data_type in data_types:
            if data_type[1] not in models.EnterpriseCustomerReportingConfiguration.MANUAL_REPORTS:
                reduced_data_types.append(data_type)
        return reduced_data_types

    @permission_required(
        'enterprise.can_access_admin_dashboard',
        fn=lambda request, enterprise_uuid: enterprise_uuid
    )
    def get(self, request, enterprise_uuid):
        """
        Get the dropdown choices for EnterpriseCustomerReportingConfiguration
        """
        enterprise_customer = get_enterprise_customer(enterprise_uuid)
        if not enterprise_customer:
            return Response({'detail': 'Could not find the enterprise customer.'}, status=HTTP_404_NOT_FOUND)

        meta = models.EnterpriseCustomerReportingConfiguration._meta
        choices = {}
        for field in meta.get_fields():
            if hasattr(field, 'choices') and field.choices:
                choices[field.name] = field.choices
        # filter out deprecated 'progress' type report versions
        data_types_for_frontend = self._get_data_types_with_recent_progress_type(list(choices.get('data_type', [])))
        # remove Pearson only reports
        choices['data_type'] = (
            self._get_data_types_for_non_pearson_customers(data_types_for_frontend)
            if 'pearson' not in enterprise_customer.slug
            else data_types_for_frontend
        )

        return Response(data=choices, status=HTTP_200_OK)



"""
Serializer for Degreed2 configuration.
"""
from rest_framework import serializers

from integrated_channels.api.serializers import EnterpriseCustomerPluginConfigSerializer
from integrated_channels.degreed2.models import Degreed2EnterpriseCustomerConfiguration


class Degreed2ConfigSerializer(EnterpriseCustomerPluginConfigSerializer):
    class Meta:
        model = Degreed2EnterpriseCustomerConfiguration
        extra_fields = (
            'encrypted_client_id',
            'encrypted_client_secret',
            'degreed_base_url',
            'degreed_token_fetch_base_url',
        )
        fields = EnterpriseCustomerPluginConfigSerializer.Meta.fields + extra_fields

    encrypted_client_id = serializers.CharField(required=False, allow_blank=False, read_only=False)
    encrypted_client_secret = serializers.CharField(required=False, allow_blank=False, read_only=False)



# Generated by Django 2.2.17 on 2020-12-08 17:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0116_auto_20201116_0400'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='enable_portal_lms_configurations_screen',
            field=models.BooleanField(default=False, help_text='Specifies whether to allow access to the external LMS configuration screen in the admin portal.'),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='enable_portal_lms_configurations_screen',
            field=models.BooleanField(default=False, help_text='Specifies whether to allow access to the external LMS configuration screen in the admin portal.'),
        ),
    ]



"""
Database models for Enterprise Integrated Channel Degreed.
"""

import json
from logging import getLogger

from config_models.models import ConfigurationModel

from django.db import models

from integrated_channels.degreed.exporters.content_metadata import DegreedContentMetadataExporter
from integrated_channels.degreed.exporters.learner_data import DegreedLearnerExporter
from integrated_channels.degreed.transmitters.content_metadata import DegreedContentMetadataTransmitter
from integrated_channels.degreed.transmitters.learner_data import DegreedLearnerTransmitter
from integrated_channels.integrated_channel.models import (
    EnterpriseCustomerPluginConfiguration,
    LearnerDataTransmissionAudit,
)
from integrated_channels.utils import is_valid_url

LOGGER = getLogger(__name__)


class DegreedGlobalConfiguration(ConfigurationModel):
    """
    The global configuration for integrating with Degreed.

    .. no_pii:
    """

    completion_status_api_path = models.CharField(
        max_length=255,
        verbose_name="Completion Status API Path",
        help_text="The API path for making completion POST/DELETE requests to Degreed."
    )

    course_api_path = models.CharField(
        max_length=255,
        verbose_name="Course Metadata API Path",
        help_text="The API path for making course metadata POST/DELETE requests to Degreed."
    )

    oauth_api_path = models.CharField(
        max_length=255,
        verbose_name="OAuth API Path",
        help_text=(
            "The API path for making OAuth-related POST requests to Degreed. "
            "This will be used to gain the OAuth access token which is required for other API calls."
        )
    )

    class Meta:
        app_label = 'degreed'

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return "<DegreedGlobalConfiguration with id {id}>".format(id=self.id)

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()


class DegreedEnterpriseCustomerConfiguration(EnterpriseCustomerPluginConfiguration):
    """
    The Enterprise-specific configuration we need for integrating with Degreed.

    .. no_pii:
    """

    key = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client ID",
        help_text=(
            "The API Client ID provided to edX by the enterprise customer to be used to make API "
            "calls to Degreed on behalf of the customer."
        )
    )

    secret = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="API Client Secret",
        help_text=(
            "The API Client Secret provided to edX by the enterprise customer to be used to make API "
            "calls to Degreed on behalf of the customer."
        )
    )

    degreed_company_id = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Degreed Organization Code",
        help_text="The organization code provided to the enterprise customer by Degreed."
    )

    degreed_base_url = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Degreed Base URL",
        help_text="The base URL used for API requests to Degreed, i.e. https://degreed.com."
    )

    degreed_user_id = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Degreed User ID",
        help_text=(
            "The Degreed User ID provided to the content provider by Degreed. "
            "It is required for getting the OAuth access token."
        )
    )

    degreed_user_password = models.CharField(
        max_length=255,
        blank=True,
        default='',
        verbose_name="Degreed User Password",
        help_text=(
            "The Degreed User Password provided to the content provider by Degreed. "
            "It is required for getting the OAuth access token."
        )
    )

    provider_id = models.CharField(
        max_length=100,
        default='EDX',
        verbose_name="Provider Code",
        help_text="The provider code that Degreed gives to the content provider."
    )

    class Meta:
        app_label = 'degreed'

    @property
    def is_valid(self):
        """
        Returns whether or not the configuration is valid and ready to be activated

        Args:
            obj: The instance of DegreedEnterpriseCustomerConfiguration
                being rendered with this admin form.
        """
        missing_items = {'missing': []}
        incorrect_items = {'incorrect': []}
        if not self.key:
            missing_items.get('missing').append('key')
        if not self.secret:
            missing_items.get('missing').append('secret')
        if not self.degreed_company_id:
            missing_items.get('missing').append('degreed_company_id')
        if not self.degreed_base_url:
            missing_items.get('missing').append('degreed_base_url')
        if not self.degreed_user_id:
            missing_items.get('missing').append('degreed_user_id')
        if not self.degreed_user_password:
            missing_items.get('missing').append('degreed_user_password')
        if not self.provider_id:
            missing_items.get('missing').append('provider_id')
        if not is_valid_url(self.degreed_base_url):
            incorrect_items.get('incorrect').append('degreed_base_url')
        if len(self.display_name) > 20:
            incorrect_items.get('incorrect').append('display_name')
        return missing_items, incorrect_items

    def __str__(self):
        """
        Return human-readable string representation.
        """
        return "<DegreedEnterpriseCustomerConfiguration for Enterprise {enterprise_name}>".format(
            enterprise_name=self.enterprise_customer.name
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    @staticmethod
    def channel_code():
        """
        Returns an capitalized identifier for this channel class, unique among subclasses.
        """
        return 'DEGREED'

    def get_learner_data_transmitter(self):
        """
        Return a ``DegreedLearnerTransmitter`` instance.
        """
        return DegreedLearnerTransmitter(self)

    def get_learner_data_exporter(self, user):
        """
        Return a ``DegreedLearnerDataExporter`` instance.
        """
        return DegreedLearnerExporter(user, self)

    def get_content_metadata_transmitter(self):
        """
        Return a ``DegreedContentMetadataTransmitter`` instance.
        """
        return DegreedContentMetadataTransmitter(self)

    def get_content_metadata_exporter(self, user):
        """
        Return a ``DegreedContentMetadataExporter`` instance.
        """
        return DegreedContentMetadataExporter(user, self)


class DegreedLearnerDataTransmissionAudit(LearnerDataTransmissionAudit):
    """
    The payload we sent to Degreed at a given point in time for an enterprise course enrollment.

    .. pii: The user_email AND degreed_user_email model fields contain PII.
    .. pii_types: email_address
    .. pii_retirement: consumer_api
    """

    degreed_user_email = models.CharField(
        max_length=255,
        blank=False,
        null=False
    )

    degreed_completed_timestamp = models.CharField(
        max_length=10,
        blank=True,
        null=True,
        help_text=(
            'Represents the Degreed representation of a timestamp: yyyy-mm-dd, '
            'which is always 10 characters.'
        )
    )

    class Meta:
        app_label = 'degreed'
        index_together = ['enterprise_customer_uuid', 'plugin_configuration_id']

    def __str__(self):
        """
        Return a human-readable string representation of the object.
        """
        return (
            '<DegreedLearnerDataTransmissionAudit {transmission_id} for enterprise enrollment {enrollment}, '
            'email {degreed_user_email}, and course {course_id}>'.format(
                transmission_id=self.id,
                enrollment=self.enterprise_course_enrollment_id,
                degreed_user_email=self.degreed_user_email,
                course_id=self.course_id
            )
        )

    def __repr__(self):
        """
        Return uniquely identifying string representation.
        """
        return self.__str__()

    def serialize(self, *args, **kwargs):
        """
        Return a JSON-serialized representation.

        Sort the keys so the result is consistent and testable.

        Can take the following keyword arguments:
            - `enterprise_configuration`: used to get the `degreed_company_id` to be sent in the payload.

        # TODO: When we refactor to use a serialization flow consistent with how course metadata
        # is serialized, remove the serialization here and make the learner data exporter handle the work.
        """
        enterprise_configuration = kwargs.get('enterprise_configuration')
        degreed_company_id = enterprise_configuration.degreed_company_id \
            if hasattr(enterprise_configuration, 'degreed_company_id') else ''
        json_payload = {
            'orgCode': degreed_company_id,
            'completions': [{
                'email': self.degreed_user_email,
                'id': self.course_id,
                'completionDate': self.degreed_completed_timestamp,
            }]
        }
        return json.dumps(json_payload, sort_keys=True)



# Generated by Django 3.2.23 on 2024-03-05 09:49

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0033_integratedchannelapirequestlogs_channel_name'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='remote_errored_at',
            field=models.DateTimeField(blank=True, help_text='Date when the content transmission was failed in the remote API.', null=True),
        ),
    ]



"""
Logging utilities for Enterprise
"""
import logging

import crum


def getEnterpriseLogger(name):
    """
    Get an Enterprise-ready logger
    """
    base_logger = logging.getLogger(name)
    return EnterpriseRequestIdLoggerAdapter(base_logger, {})


def get_request_id():
    """
    Helper to get the request id - usually set via an X-Request-ID header
    """
    request = crum.get_current_request()
    if request is not None and request.headers is not None:
        return request.headers.get('X-Request-ID')
    else:
        return None


class EnterpriseRequestIdLoggerAdapter(logging.LoggerAdapter):
    """
    A utility for logging X-Request-ID information
    https://docs.python.org/3/howto/logging-cookbook.html#using-loggeradapters-to-impart-contextual-information
    """
    def process(self, msg, kwargs):
        if request_id := get_request_id():
            msg = f'[request_id {request_id}] {msg}'
        return msg, kwargs



# Generated by Django 2.2.17 on 2021-02-10 09:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0122_remove_field_sync_enterprise_catalog_query'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomeridentityprovider',
            name='default_provider',
            field=models.BooleanField(default=False, help_text='Specifies whether this is default provider for enterprise customer.'),
        ),
    ]



"""
Serializer for integrated channel api.
"""

from rest_framework import serializers

from integrated_channels.integrated_channel.models import EnterpriseCustomerPluginConfiguration


class EnterpriseCustomerPluginConfigSerializer(serializers.ModelSerializer):
    """
    Serializer for EnterpriseCustomerPluginConfiguration model
    """
    is_valid = serializers.ReadOnlyField()
    channel_code = serializers.ReadOnlyField()
    last_sync_attempted_at = serializers.ReadOnlyField()
    last_content_sync_attempted_at = serializers.ReadOnlyField()
    last_learner_sync_attempted_at = serializers.ReadOnlyField()
    last_sync_errored_at = serializers.ReadOnlyField()
    last_content_sync_errored_at = serializers.ReadOnlyField()
    last_learner_sync_errored_at = serializers.ReadOnlyField()
    last_modified_at = serializers.ReadOnlyField()

    class Meta:
        model = EnterpriseCustomerPluginConfiguration
        fields = (
            'id',
            'display_name',
            'channel_code',
            'enterprise_customer',
            'idp_id',
            'active',
            'is_valid',
            'last_sync_attempted_at',
            'last_content_sync_attempted_at',
            'last_learner_sync_attempted_at',
            'last_sync_errored_at',
            'last_content_sync_errored_at',
            'last_learner_sync_errored_at',
            'transmission_chunk_size',
            'last_modified_at',
        )



# Generated by Django 4.2.15 on 2024-08-08 17:29

import collections
from django.db import migrations
import enterprise.validators
import jsonfield.encoder
import jsonfield.fields


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0218_add_provisioning_admin_role'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecatalogquery',
            name='content_filter',
            field=jsonfield.fields.JSONField(blank=True, default={}, dump_kwargs={'cls': jsonfield.encoder.JSONEncoder, 'indent': 4, 'separators': (',', ':')}, help_text="Query parameters which will be used to filter the discovery service's search/all endpoint results, specified as a JSON object. An empty JSON object means that all available content items will be included in the catalog.  Must be unique.", load_kwargs={'object_pairs_hook': collections.OrderedDict}, null=True, validators=[enterprise.validators.validate_content_filter_fields]),
        ),
    ]



# Generated by Django 3.2.15 on 2023-01-05 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('degreed2', '0019_alter_degreed2learnerdatatransmissionaudit_index_together'),
    ]

    operations = [
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_content_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Content data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_content_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Content data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_learner_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_learner_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Content or Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='degreed2enterprisecustomerconfiguration',
            name='last_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Content or Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_content_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Content data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_content_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Content data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_learner_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_learner_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_sync_attempted_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent Content or Learner data record sync attempt', null=True),
        ),
        migrations.AddField(
            model_name='historicaldegreed2enterprisecustomerconfiguration',
            name='last_sync_errored_at',
            field=models.DateTimeField(blank=True, help_text='The DateTime of the most recent failure of a Content or Learner data record sync attempt', null=True),
        ),
    ]



# Generated by Django 3.2.11 on 2022-01-26 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('moodle', '0008_alter_moodleenterprisecustomerconfiguration_enterprise_customer'),
    ]

    operations = [
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='catalogs_to_transmit',
            field=models.TextField(blank=True, default='', help_text='A comma-separated list of catalog UUIDs to transmit. If blank, all customer catalogs will be transmitted. If there are overlapping courses in the customer catalogs, the overlapping course metadata will be selected from the newest catalog.'),
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, default='', help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255),
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, default='', help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255),
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='moodle_base_url',
            field=models.CharField(blank=True, help_text='The base URL used for API requests to Moodle', max_length=255, verbose_name='Moodle Base URL'),
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='password',
            field=models.CharField(blank=True, default='', help_text="The API user's password used to obtain new tokens.", max_length=255, verbose_name='Webservice Password'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='service_short_name',
            field=models.CharField(blank=True, help_text='The short name for the Moodle webservice.', max_length=255, verbose_name='Webservice Short Name'),
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='token',
            field=models.CharField(blank=True, default='', help_text="The user's token for the Moodle webservice.", max_length=255, verbose_name='Webservice User Token'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='historicalmoodleenterprisecustomerconfiguration',
            name='username',
            field=models.CharField(blank=True, default='', help_text="The API user's username used to obtain new tokens.", max_length=255, verbose_name='Webservice Username'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='catalogs_to_transmit',
            field=models.TextField(blank=True, default='', help_text='A comma-separated list of catalog UUIDs to transmit. If blank, all customer catalogs will be transmitted. If there are overlapping courses in the customer catalogs, the overlapping course metadata will be selected from the newest catalog.'),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='channel_worker_username',
            field=models.CharField(blank=True, default='', help_text='Enterprise channel worker username to get JWT tokens for authenticating LMS APIs.', max_length=255),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='idp_id',
            field=models.CharField(blank=True, default='', help_text='If provided, will be used as IDP slug to locate remote id for learners', max_length=255),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='moodle_base_url',
            field=models.CharField(blank=True, help_text='The base URL used for API requests to Moodle', max_length=255, verbose_name='Moodle Base URL'),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='password',
            field=models.CharField(blank=True, default='', help_text="The API user's password used to obtain new tokens.", max_length=255, verbose_name='Webservice Password'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='service_short_name',
            field=models.CharField(blank=True, help_text='The short name for the Moodle webservice.', max_length=255, verbose_name='Webservice Short Name'),
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='token',
            field=models.CharField(blank=True, default='', help_text="The user's token for the Moodle webservice.", max_length=255, verbose_name='Webservice User Token'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='moodleenterprisecustomerconfiguration',
            name='username',
            field=models.CharField(blank=True, default='', help_text="The API user's username used to obtain new tokens.", max_length=255, verbose_name='Webservice Username'),
            preserve_default=False,
        ),
    ]



# Generated by Django 3.2.23 on 2024-08-26 09:54

from django.db import migrations
from integrated_channels.sap_success_factors.utils import populate_decrypted_fields_sap_success_factors


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0021_sapsuccessfactorsenterprisecustomerconfiguration_decrypted_secret'),
    ]

    operations = [
        migrations.RunPython(populate_decrypted_fields_sap_success_factors, reverse_code=migrations.RunPython.noop),
    ]



"""
Views for the ``enterprise-user`` API endpoint.
"""

from collections import OrderedDict

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import filters, permissions, response, status
from rest_framework.pagination import PageNumberPagination

from django.contrib import auth
from django.core.exceptions import ValidationError
from django.db.models import Q

from enterprise import models
from enterprise.api.v1 import serializers
from enterprise.api.v1.views.base_views import EnterpriseReadOnlyModelViewSet
from enterprise.logging import getEnterpriseLogger

User = auth.get_user_model()

LOGGER = getEnterpriseLogger(__name__)


class EnterpriseCustomerSupportPaginator(PageNumberPagination):
    """Custom paginator for the enterprise customer support."""

    page_size = 8

    def get_paginated_response(self, data):
        """Return a paginated style `Response` object for the given output data."""
        return response.Response(
            OrderedDict(
                [
                    ("count", self.page.paginator.count),
                    ("num_pages", self.page.paginator.num_pages),
                    ("next", self.get_next_link()),
                    ("previous", self.get_previous_link()),
                    ("results", data),
                ]
            )
        )

    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a page object,
        or `None` if pagination is not configured for this view.

        """
        if isinstance(queryset, filter):
            queryset = list(queryset)

        return super().paginate_queryset(queryset, request, view)


class EnterpriseCustomerSupportViewSet(EnterpriseReadOnlyModelViewSet):
    """
    API views for the ``enterprise-customer-support`` API endpoint.
    """

    queryset = models.PendingEnterpriseCustomerUser.objects.all()
    filter_backends = (DjangoFilterBackend, filters.OrderingFilter)
    permission_classes = (permissions.IsAuthenticated,)
    paginator = EnterpriseCustomerSupportPaginator()

    ordering_fields = ["is_admin", "username", "pending_enterprise_customer"]

    def filter_queryset_by_user_query(self, queryset, is_pending_user=False):
        """
        Filter queryset based on user provided query
        """
        user_query = self.request.query_params.get("user_query", None)

        if user_query:
            if not is_pending_user:
                queryset = queryset.filter(
                    user_id__in=User.objects.filter(
                        Q(email__icontains=user_query) | Q(username__icontains=user_query)
                    )
                )
            else:
                queryset = queryset.filter(user_email=user_query)

        return queryset

    def retrieve(self, request, *args, **kwargs):
        """
        Filter down the queryset of groups available to the requesting uuid.
        """
        enterprise_uuid = kwargs.get("enterprise_uuid", None)
        users = []

        try:
            enterprise_customer_queryset = models.EnterpriseCustomerUser.objects.filter(
                enterprise_customer__uuid=enterprise_uuid,
            )
            enterprise_customer_queryset = self.filter_queryset_by_user_query(
                enterprise_customer_queryset
            )
            users.extend(enterprise_customer_queryset)

            pending_enterprise_customer_queryset = (
                models.PendingEnterpriseCustomerUser.objects.filter(
                    enterprise_customer__uuid=enterprise_uuid
                ).order_by("user_email")
            )
            pending_enterprise_customer_queryset = self.filter_queryset_by_user_query(
                pending_enterprise_customer_queryset, is_pending_user=True
            )
            users.extend(pending_enterprise_customer_queryset)

        except ValidationError:
            # did not find UUID match in either EnterpriseCustomerUser or PendingEnterpriseCustomerUser
            return response.Response(
                {"detail": "Could not find enterprise uuid {}".format(enterprise_uuid)},
                status=status.HTTP_404_NOT_FOUND,
            )

        # default sort criteria
        is_reversed = False

        ordering_criteria = self.request.query_params.get("ordering", None)
        if ordering_criteria:
            is_reversed = '-' in ordering_criteria

        # paginate the queryset
        users_page = self.paginator.paginate_queryset(users, request, view=self)

        # serialize the paged dataset
        serializer = serializers.EnterpriseUserSerializer(users_page, many=True)
        serializer_data = serializer.data
        # apply pre-serialization ordering by user criteria before the users
        # get divvied up by pagination
        if ordering_criteria in ("administrator", "learner"):
            serializer_data = sorted(
                serializer_data,
                key=lambda k: (
                    (k["is_admin"])
                ),
                reverse=is_reversed
            )
        if ordering_criteria == "details":
            serializer_data = sorted(
                serializer_data,
                key=lambda k: (
                    (
                        (
                            k["enterprise_customer_user"]["username"]
                            if k["enterprise_customer_user"] is not None
                            else k["pending_enterprise_customer_user"]["user_email"]
                        ),
                    )
                ),
                reverse=is_reversed,
            )
        # Apply post-serialization default ordering criteria (first by is_admin,
        # then first name) only if user does not specify ordering criteria;
        # Process this after the data has been serialized since the is_admin
        # field is computed/available only after serialization step
        if not ordering_criteria:
            serializer_data = sorted(
                serializer_data,
                key=lambda k: (
                    # sort by is_admin = True first (i.e. -1),
                    # then sort by first_name lexicographically
                    (
                        -k["is_admin"],
                        (
                            k["enterprise_customer_user"]["username"]
                            if k["enterprise_customer_user"] is not None
                            else k["pending_enterprise_customer_user"]["user_email"]
                        ),
                    )
                ),
            )
        return self.paginator.get_paginated_response(serializer_data)
